/*	What is the difference between evs.c and evs2.c?  Evs2.c is used to build 
 * 	the 5206AN board.
 * File:		evs.c
 * Purpose:		Board specific routines for Arnewsh 5206 EVS
 *
 * Notes:
 *
 * Date:		7-3-96
 *
 * Modifications:
 *
 */


#include "uif/dbug.h"
#include "uif/isr.h"
#include "device/mc68hc901.h"
#include "device/amd29f010.h"
#include "device/ns8390.h"

#ifdef NETWORK
#include "net/nif.h"
#include "net/ip.h"
#include "net/arp.h"

/*
 * Define one network interface for TFTP.
 */
NIF nif1;

PROTO_IP_INFO
ip_info1;

PROTO_ARP_INFO
arp_info1;

#endif

const char BRDSTR[] = "EVS";
const int  BRDVER   = 1;

#define FILENAME_SIZE	(80)

/*
 * The AMD29F010 FLASH devices.  On this board, we've got two
 * 8-bit wide devices acting like a single 16-bit device.  This
 * gives 256K of flash on board.
 */
typedef struct
{
	NATURAL32	baud;		/* baud rate of the board */
	BYTE	client[4];		/* board IP address */
	BYTE	server[4];		/* network download host IP address */
	BYTE	gateway[4];		/* gateway IP address */
	BYTE	netmask[4];		/* netmask address */
	BYTE	netcfg[4];		/* network download config */
		/*
		 * netcfg[0] is autoboot flag
		 * netcfg[1] is number of seconds before autoboot
		 * netcfg[2] is ???
		 * netcfg[3] is ???
		 */
	NATURAL32	nicbase;
	NATURAL32	nicirq;
	BYTE	filename[FILENAME_SIZE];	/* network download filename */
	NATURAL32	dram_width;	/* 8, 16 or 32 */
} PARAM;

typedef struct
{
	BYTE	data[AMD29F010_SECTOR_SIZE * 2];
} SECTOR;

typedef struct
{
	SECTOR	dbug;		/* allow 96K for this debug monitor */
	SECTOR	dbug1;
	SECTOR	dbug2;
	SECTOR	param;		/* 32K of config info */
	SECTOR	sector4;
	SECTOR	sector5;
	SECTOR	sector6;
	SECTOR	sector7;
} FLASH;


typedef struct
{
	NS8390		*nic;		/* NE 2000 */
	MC68HC901	*mc68hc901;
	FLASH	  	*flash;
	int		(*flash_write)(AMD29F010 *, void *, int, int, void (*)(void));
	unsigned	nicirq;
	unsigned	dram_size;
} BOARD;

/*
 * Here is the primary data structure which contains pointers to
 * the peripherals contained on the evaluation board.
 */
BOARD	board;

/*************************************************************/
void
board_init (void)
{
	extern ADDRESS VECTOR_TABLE[];
	extern char __DATA_ROM[];
	extern char __DATA_RAM[];
	extern char __DATA_END[];
	extern char __BSS_START[];
	extern char __BSS_END[];
	extern char __FCR_WRITE[];
	extern REGISTERS context;
	int probe (NATURAL32, NATURAL32);
	int vector;
	NATURAL32 nicbase, nicirq, nicOK;
	MCF5206_IMM *imm;

    /*
     * Point VBR to the vector table - MUST be aligned to 1M boundary
     */
    mcf5206_write_vbr((NATURAL32)VECTOR_TABLE);
 
    /*
     * Invalidate the cache and disable it
     */
    mcf5206_write_cacr (MCF5206_CACR_CINV);

    /*
     * Setup ACRs so that if cache turned on, periphal accesses
     * are not messed up.  (Non-cacheable, serialized)
     */
    mcf5206_write_acr0 ( 0
        | MCF5206_ACR_BASE(MC68HC901_ADDRESS)
        | MCF5206_ACR_MASK(0x0F000000)
        | MCF5206_ACR_EN
        | MCF5206_ACR_S_IGNORE
    );
    mcf5206_write_acr1 ( 0
        | MCF5206_ACR_BASE(MC68HC901_ADDRESS)
        | MCF5206_ACR_MASK(0x0F000000)
        | MCF5206_ACR_EN
        | MCF5206_ACR_S_IGNORE
    );

	/*
	 * Setup SRAMBAR and MBAR
	 */
    mcf5206_write_srambar ( 0
		| MCF5206_SRAMBAR_BASE(SRAM_ADDRESS)
		| MCF5206_SRAMBAR_AS_CI
		| MCF5206_SRAMBAR_V
		) ;

    mcf5206_write_mbar ( 0
		| MCF5206_MBAR_BASE(IMM_ADDRESS)
		| MCF5206_MBAR_V
		) ;

	/*
	 * Copy vector table to RAM address 0x00000000.
	 */
	for (vector = 0; vector < 256; ++vector)
	{
		*(ADDRESS *)(vector << 2) = VECTOR_TABLE[vector];
	}
	*(ADDRESS *)(0) = 0;	/* good for detecting NULL dereferences */

	/* move initialized data from ROM to RAM if defined at		*/
	/* different locations.										*/
	if (__DATA_ROM != __DATA_RAM)
	{
		memcpy(__DATA_RAM,__DATA_ROM,__DATA_END-__DATA_RAM);
	}
 
	/* clear uninitialized data             */
	if (__BSS_START != __BSS_END)
	{
		memset(__BSS_START,0,__BSS_END-__BSS_START);
	}

	/*
	 * The following values are defined by the memory map.
	 */
	imm = mcf5206_get_immp();
	board.mc68hc901 =	(MC68HC901 *)MC68HC901_ADDRESS;
	board.flash =		(FLASH *)FLASH_ADDRESS;
	board.flash_write = (void *)__FCR_WRITE;

	/*
	 * Setup MC68HC901.  This requires setting up I/O pins for ISA
	 * IRQs and initializing the timers and UART for serial IO.
	 *
	 * All general purpose I/O pins are configured as inputs for
	 * ISA IRQs.  ISA IRQs detected on rising edge.
	 */
	board.mc68hc901->VR = ( 0
		| MC68HC901_VR_VECTOR(MC68HC901_IRQ)
		) ;

	board.mc68hc901->DDR = ( ~0
		& MC68HC901_DDR_GPIP7_INPUT		/* ISA IRQ 11 */
		& MC68HC901_DDR_GPIP6_INPUT		/* ISA IRQ 10 */
		& MC68HC901_DDR_GPIP5_INPUT		/* ISA IRQ 9  */
		& MC68HC901_DDR_GPIP4_INPUT		/* ISA IRQ 7  */
		& MC68HC901_DDR_GPIP3_INPUT		/* ISA IRQ 6  */
		& MC68HC901_DDR_GPIP2_INPUT		/* ISA IRQ 5  */
		& MC68HC901_DDR_GPIP1_INPUT		/* ISA IRQ 4  */
		& MC68HC901_DDR_GPIP0_INPUT		/* ISA IRQ 3  */
		) ;

	board.mc68hc901->AER = ( 0
		| MC68HC901_AER_GPIP7_RISE
		| MC68HC901_AER_GPIP6_RISE
		| MC68HC901_AER_GPIP5_RISE
		| MC68HC901_AER_GPIP4_RISE
		| MC68HC901_AER_GPIP3_RISE
		| MC68HC901_AER_GPIP2_RISE
		| MC68HC901_AER_GPIP1_RISE
		| MC68HC901_AER_GPIP0_RISE
		) ;

	/*
	 * Initialize MC68HC901 timers.  Timer D is routed into the
	 * Receive and Transmit Clocks on the MC68HC901 to generate
	 * baud rates.  XTAL is 2.4576MHz
	 */
	board.mc68hc901->TACR = MC68HC901_TCR_MODE_STOP;
	board.mc68hc901->TBCR = MC68HC901_TCR_MODE_STOP;
	board.mc68hc901->TCDCR = ( 0
		| MC68HC901_TCDCR_TC_MODE_STOP
		| MC68HC901_TCDCR_TD_MODE_STOP
		) ;

	switch (board_get_baud())
	{
		case  9600:
			board.mc68hc901->TDDR = 2;
			board.mc68hc901->TCDCR = ( 0
				/* yields 9600 * 16 clk into RC/TC */
				| MC68HC901_TCDCR_TD_MODE_DM_DIV4
				) ;
			break;
		default:
		case 19200:
			board.mc68hc901->TDDR = 1;
			board.mc68hc901->TCDCR = ( 0
				/* yields 19200 * 16 clk into RC/TC */
				| MC68HC901_TCDCR_TD_MODE_DM_DIV4
				) ;
			break;
	}

	mc68hc901_uart_init (board.mc68hc901);

	/*
	 * Install the NE2000 network card interrupt handler
	 */
	isr_init();
	nicOK = TRUE;
	vector = board_get_nicirq();
	switch (vector)
	{
		case 11:
			board.mc68hc901->IERB = 0;
			board.mc68hc901->IERA = MC68HC901_IERA_GPIP7;
			board.mc68hc901->IMRB = 0;
			board.mc68hc901->IMRA = MC68HC901_IMRA_GPIP7;
			nicirq = MC68HC901_VR_IV_GPI7;
			break;
		case 10:
			board.mc68hc901->IERB = 0;
			board.mc68hc901->IERA = MC68HC901_IERA_GPIP6;
			board.mc68hc901->IMRB = 0;
			board.mc68hc901->IMRA = MC68HC901_IMRA_GPIP6;
			nicirq = MC68HC901_VR_IV_GPI6;
			break;
		case 9:
			board.mc68hc901->IERA = 0;
			board.mc68hc901->IERB = MC68HC901_IERB_GPIP5;
			board.mc68hc901->IMRA = 0;
			board.mc68hc901->IMRB = MC68HC901_IMRB_GPIP5;
			nicirq = MC68HC901_VR_IV_GPI5;
			break;
		case 7:
			board.mc68hc901->IERA = 0;
			board.mc68hc901->IERB = MC68HC901_IERB_GPIP4;
			board.mc68hc901->IMRA = 0;
			board.mc68hc901->IMRB = MC68HC901_IMRB_GPIP4;
			nicirq = MC68HC901_VR_IV_GPI4; 
			break;
		case 6:
			board.mc68hc901->IERA = 0;
			board.mc68hc901->IERB = MC68HC901_IERB_GPIP3;
			board.mc68hc901->IMRA = 0;
			board.mc68hc901->IMRB = MC68HC901_IMRB_GPIP3;
			nicirq = MC68HC901_VR_IV_GPI3;
			break;
		case 5:
			board.mc68hc901->IERA = 0;
			board.mc68hc901->IERB = MC68HC901_IERB_GPIP2;
			board.mc68hc901->IMRA = 0;
			board.mc68hc901->IMRB = MC68HC901_IMRB_GPIP2;
			nicirq = MC68HC901_VR_IV_GPI2;
			break;
		case 4:
			board.mc68hc901->IERA = 0;
			board.mc68hc901->IERB = MC68HC901_IERB_GPIP1;
			board.mc68hc901->IMRA = 0;
			board.mc68hc901->IMRB = MC68HC901_IMRB_GPIP1;
			nicirq = MC68HC901_VR_IV_GPI1;
			break;
		case 3:
			board.mc68hc901->IERA = 0;
			board.mc68hc901->IERB = MC68HC901_IERB_GPIP0;
			board.mc68hc901->IMRA = 0;
			board.mc68hc901->IMRB = MC68HC901_IMRB_GPIP0;
			nicirq = MC68HC901_VR_IV_GPI0;
			break;
		default:
			nicOK = FALSE;
			break;
	}
	nicirq += MC68HC901_IRQ;

	nicbase = board_get_nicbase() + ISA_ADDRESS;

	if (nicOK)
	{
		board.nic = (NS8390 *)nicbase;
		board.nicirq = nicirq;
	}
	else
	{
		board.nic = NULL;
		board.nicirq = 0;
	}

	/*
	 * Clear out any MC68HC901 & ISA interrupts.
	 */
	board.mc68hc901->IPRA = 0;
	board.mc68hc901->IPRB = 0;
	board.mc68hc901->ISRA = 0;
	board.mc68hc901->ISRB = 0;

	/*
	 * Enable NMI interrupt to ColdFire core
	 */
	imm->sim.IMR &= ~MCF5206_SIM_IMR_EINT7;

	switch (imm->sim.RSR)
	{
		case MCF5206_SIM_RSR_HRST:
			printf("\n\nHard Reset\n");
			break;
		case MCF5206_SIM_RSR_SWTR:
			printf("\n\nSoftware Watchdog Reset\n");
			break;
	}
	imm->sim.RSR = ( 0
		| MCF5206_SIM_RSR_HRST
		| MCF5206_SIM_RSR_SWTR
		) ;

	/*
	 * Determine size of DRAM Bank 0
	 */
	imm->dramc.DCMR0 = MCF5206_DRAMC_DCMR_MASK_16M;
	if (probe(DRAM_ADDRESS,0x00800000))
	{
		board.dram_size = 0x01000000;
	}
	else
	{
		imm->dramc.DCMR0 = MCF5206_DRAMC_DCMR_MASK_4M;
		if (probe(DRAM_ADDRESS,0x00100000))
		{
			board.dram_size = 0x00400000;
		}
		else
		{
			imm->dramc.DCMR0 = MCF5206_DRAMC_DCMR_MASK_1M;
			board.dram_size = 0x00100000;
		}
	}

	/*
	 * Now check DRAM Bank 1 for existance by enabling identical
	 * to DRAM Bank 0, and probing
	 */
	imm->dramc.DCCR1 = imm->dramc.DCCR0;
	imm->dramc.DCMR1 = imm->dramc.DCMR0;
	imm->dramc.DCAR1 = MCF5206_DRAMC_DCAR_BASE(board.dram_size);
	if (!probe(DRAM_ADDRESS+board.dram_size,4))
	{
		/* not there, disable it */
		imm->dramc.DCCR1 = 0;
		imm->dramc.DCMR1 = 0;
		imm->dramc.DCAR1 = 0;
	}
	else
	{
		board.dram_size <<= 1;
	}

	printf("DRAM Size: %dM\n",board.dram_size>>20);
	context.a7 = board.dram_size;

	if (nicOK)
	{
		printf("NE2000: %#03X, IRQ %d\n", nicbase & 0x00000FE0, vector);
	}
}

/*************************************************************/
static void
copy_flash_driver_to_dram (void)
{
	extern unsigned int __FCR_START[];
	extern unsigned int __FCR_SIZE[];   /* given in BYTES */
	extern unsigned int __FLC_START[];
	extern char __FCR_WRITE[];
 
	int i;
 
	for (i = 0; i < ((int)__FCR_SIZE >> 2); ++i)
	{
	    __FCR_START[i] = __FLC_START[i];
	}
	board.flash_write = (void *)__FCR_WRITE;
}

/*************************************************************/
static void
copy_flash_sector_to_dram (NATURAL32 *sector)
{
	/*
	 * This code copies ONE sector from the flash into DRAM
	 * pointed to by target.
	 */
	int i;
	NATURAL32 *target = (NATURAL32 *)__USER_SPACE;
 
	for (i = 0; i < AMD29F010_SECTOR_SIZE; ++i)
	{
		target[i] = sector[i];
	}
}

/*************************************************************/
static void
update_board_params (BYTE *data, int length, NATURAL32 offset)
{
	/*
	 * This function copies the PARAM section of the flash
	 * into DRAM, modifies the DRAM copy of PARAM with 'data'
	 * for 'length' bytes, then writes the updated copy into
	 * flash again.
	 */
	BYTE *target;
	int index, bbytes;
 
	board_disable_interrupts();
 
	copy_flash_sector_to_dram((NATURAL32 *)&board.flash->param);
 
	target = (BYTE *)(__USER_SPACE + offset);
	for (index = 0; index < length; ++index)
	{
		*target++ = *data++;
	}
 
	copy_flash_driver_to_dram();
 
	bbytes = board.flash_write((AMD29F010 *)board.flash,
		(void *)__USER_SPACE, sizeof(PARAM), 3, NULL);
 
	if (bbytes != sizeof(PARAM))
	{
		printf("Flash programming error:  %d bytes of %d written!\n",
			bbytes, sizeof(PARAM));
	}
}

/*************************************************************/
void
board_set_baud (int baud)
{
	unsigned char bps[4];
	PARAM   *tparam = (PARAM *)__USER_SPACE;
 
	bps[0] = ((baud & 0xFF000000) >> 24);
	bps[1] = ((baud & 0x00FF0000) >> 16);
	bps[2] = ((baud & 0x0000FF00) >>  8);
	bps[3] = ((baud & 0x000000FF) >>  0);
 
	update_board_params(bps, 4,
		(NATURAL32)((NATURAL32)&tparam->baud - (NATURAL32)__USER_SPACE));
}

/*************************************************************/
int
board_get_baud (void)
{
	return ((PARAM *)&board.flash->param)->baud;
}

/*************************************************************/
int
board_vda (ADDRESS addr)
{
	/*
	 * board_vda determines if the given address is a valid
	 * address for downloading code.  This routine is called
	 * by routines for S-record, COFF, ELF, and Image
	 * downloading.
	 */

    /*
     * Don't want download clobbering dBUG memory space.
     */
	if ((addr >= (ADDRESS)__USER_SPACE) && (addr < board.dram_size))
		return 1;
	else
		return 0;
}

/*************************************************************/
void
board_reset (void)
{
}

#ifdef NETWORK
/*************************************************************/
void
board_set_client (unsigned char client[])
{
    PARAM   *tparam = (PARAM *)__USER_SPACE;
 
    update_board_params(client, 4,
        (NATURAL32)((NATURAL32)&tparam->client - (NATURAL32)__USER_SPACE));
}

/*************************************************************/
unsigned char *
board_get_client (unsigned char client[])
{
	int i;
 
	for (i = 0; i < 4; i++)
	{
		client[i] = ((PARAM *)&(board.flash->param))->client[i];
	}
	return client;
}

/*************************************************************/
void
board_set_server (unsigned char server[])
{
    PARAM   *tparam = (PARAM *)__USER_SPACE;
 
    update_board_params(server, 4,
        (NATURAL32)((NATURAL32)&tparam->server - (NATURAL32)__USER_SPACE));
}

/*************************************************************/
unsigned char *
board_get_server (unsigned char server[])
{
	int i;
 
	for (i = 0; i < 4; i++)
	{
		server[i] = ((PARAM *)&(board.flash->param))->server[i];
	}
	return server;
}

/*************************************************************/
void
board_set_gateway (unsigned char gateway[])
{
    PARAM   *tparam = (PARAM *)__USER_SPACE;
 
    update_board_params(gateway, 4,
        (NATURAL32)((NATURAL32)&tparam->gateway - (NATURAL32)__USER_SPACE));
}

/*************************************************************/
unsigned char *
board_get_gateway (unsigned char gateway[])
{
	int i;
 
	for (i = 0; i < 4; i++)
	{
		gateway[i] = ((PARAM *)&(board.flash->param))->gateway[i];
	}
	return gateway;
}

/*************************************************************/
void
board_set_netmask (unsigned char netmask[])
{
    PARAM   *tparam = (PARAM *)__USER_SPACE;
 
    update_board_params(netmask, 4,
        (NATURAL32)((NATURAL32)&tparam->netmask - (NATURAL32)__USER_SPACE));
}

/*************************************************************/
unsigned char *
board_get_netmask (unsigned char netmask[])
{
	int i;
 
	for (i = 0; i < 4; i++)
	{
		netmask[i] = ((PARAM *)&(board.flash->param))->netmask[i];
	}
	return netmask;
}

/*************************************************************/
void
board_set_filename (char filename[])
{
    PARAM   *tparam = (PARAM *)__USER_SPACE;

    update_board_params((BYTE *)filename, FILENAME_SIZE,
        (NATURAL32)((NATURAL32)&tparam->filename - (NATURAL32)__USER_SPACE));
}

/*************************************************************/
char *
board_get_filename (char filename[])
{
	int c, i = 0;
 
	while (i < FILENAME_SIZE)
	{
		c = ((PARAM *)&board.flash->param)->filename[i];
		filename[i] = c;
		++i;
		if (c == '\0')
		{
			i = FILENAME_SIZE;
		}
	}
	filename[i] = '\0';
	return filename;
}

/*************************************************************/
int
board_get_filetype (void)
{
	return ((PARAM *)&board.flash->param)->netcfg[2];
}

/*************************************************************/
void
board_set_filetype (int ft)
{
    PARAM   *tparam = (PARAM *)__USER_SPACE;
    BYTE netcfg;
 
    netcfg = ft;
 
    update_board_params(&netcfg, 1,
        (NATURAL32)((NATURAL32)&tparam->netcfg[2] - (NATURAL32)__USER_SPACE));
}

/*************************************************************/
int
board_get_autoboot (void)
{
	return ((PARAM *)&board.flash->param)->netcfg[0];
}

/*************************************************************/
void
board_set_autoboot (int onoff)
{
    PARAM *tparam = (PARAM *)__USER_SPACE;
    BYTE netcfg;
 
    if (onoff)
    {
        netcfg = 1;
    }
    else
    {
        netcfg = 0;
    }
 
    update_board_params(&netcfg, 1,
        (NATURAL32)((NATURAL32)&tparam->netcfg[0] - (NATURAL32)__USER_SPACE));
}
#endif	/* NETWORK */

/*************************************************************/
void
updbug (int argc, char **argv)
{
	char answer[MAX_LINE];
	printf("If the new dBUG image is present at %#08X, and you wish to\n",
		__USER_SPACE);
	printf("continue, enter 'UPDATE'.\n");

	printf("Response:  ");
	get_line(answer);
	if (strcmp("UPDATE",answer) == 0)
	{
		board_disable_interrupts();
		copy_flash_driver_to_dram();

		board.flash_write((AMD29F010 *)board.flash,
			(void *)__USER_SPACE, (sizeof(SECTOR) * 3),
			AMD29F010_SECTOR_0, (void *)0xFFE00400);
	}
}

/*************************************************************/
void
upuser (int argc, char **argv)
{
	char answer[MAX_LINE];
	printf("If the new USER image is present at %#08X, and you wish to\n",
		__USER_SPACE);
	printf("continue, enter 'UPDATE'.\n");

	printf("Response:  ");
	get_line(answer);
	if (strcmp("UPDATE",answer) == 0)
	{
		board_disable_interrupts();
		copy_flash_driver_to_dram();

		board.flash_write((AMD29F010 *)board.flash,
			(void *)__USER_SPACE, (sizeof(SECTOR) * 4),
			AMD29F010_SECTOR_4, NULL);
	}
}

/*************************************************************/
/*************************************************************/
/*************************************************************/
/*************************************************************/
/*************************************************************/

void
board_enable_interrupts (void)
{
	asm_set_ipl (0);
}

void
board_disable_interrupts (void)
{
	asm_set_ipl (6);
}

/*************************************************************/
int
board_in_char (void)
{
	return mc68hc901_uart_in_char (board.mc68hc901);
}

/*************************************************************/
void
board_out_char (int c)
{
	mc68hc901_uart_out_char (board.mc68hc901, c);
}

/*************************************************************/
int
board_char_present (void)
{
	return mc68hc901_uart_char_present (board.mc68hc901);
}

/*************************************************************/
void
board_out_char_flush (void)
{
	/* No flushing necessary */
}

/*************************************************************/
int
board_next_char (int iosource)
{
	int c;
#ifdef NETWORK
	extern int tftp_in_char (void);
#endif

	switch (iosource)
	{
#ifdef NETWORK
		case DL_TFTP:
			c = tftp_in_char();
			break;
#endif
		case DL_SERIAL:
		default:
			c = board_in_char();
			break;
	}
	return c;
}

/*************************************************************/
void
board_autoboot (void)
{
}

/*************************************************************/
#ifdef NETWORK
int
board_nic_init (void)
{
	IP_ADDR Client;
	IP_ADDR Gateway;
	IP_ADDR Netmask;
	MCF5206_IMM *imm = mcf5206_get_immp();

	if (board.nic == NULL)
	{
		printf("Error: NIC not configured\n");
		return -1;
	}

	if (!isr_register_handler(ISR_DBUG_ISR, board.nicirq,
			(void *)ns8390_handler,(void *)board.nic, (void *)&nif1))
	{
		printf("Error: Unable to register handler\n");
		return -1;
	}

	/*
	 * Enable MC68HC901 interrupt to ColdFire core
	 */
	imm->sim.IMR &= ~MCF5206_SIM_IMR_EINT4;

	ns8390_init (&nif1, board.nic, MC68HC901_IRQ+MC68HC901_VR_IV_GPI2);

	board_get_client(Client);
	board_get_gateway(Gateway);
	board_get_netmask(Netmask);

	proto_ip_config(&ip_info1,Client,Gateway,Netmask);
	nif_bind_proto(&nif1,FRAME_IP,(void *)ip_handler,(void *)&ip_info1);

	proto_arp_config(&arp_info1);
	nif_bind_proto(&nif1,FRAME_ARP,(void *)arp_handler,(void *)&arp_info1);

	/*
	 * The ISA bus runs at 6-8MHz, starving the MCF5206. As a result
	 * the MCF5206 needs all the help it can get for improving its
	 * utilization of the bus.  Decreasing the default wait states for
	 * the Flash improve bus utilization, but those are changeable
	 * by the user.  Therefore, we turn the cache on!
	 * When caches were not turned on, the NS8390 receiver would fill
	 * up, causing a Receiver Buffer Overflow, and shutting down the
	 * entire card.  Same for when wait-states were not set at an
	 * optimal setting.
	 *
	 * Caches are turned off at the end of a download, via normal
	 * cpu_icache_flush() calls.
	 *
	 * WARNING!!!! See note about ICACHE in board_nic_done() below!
	 */
	mcf5206_write_cacr(MCF5206_CACR_CENB);

	return 1;
}

/*************************************************************/
void
board_nic_done (void)
{
	MCF5206_IMM *imm = mcf5206_get_immp();

#if 0
	/*
	 * For reasons unknown to me at this time, the MCF5206
	 * requires two CACR_CINV in order to shut off ICACHE
	 * properly.  The first is performed by the DN command
	 * when finished downloading, and the second one is
	 * performed right here.
	 *
	 * So, if the ICACHE is turned on in board_nic_init(),
	 * then the following code is necessary, or else
	 * sporadic exceptions results due to ICACHE fetching
	 * errant information.
	 */
	mcf5206_write_cacr(MCF5206_CACR_CINV);
#endif

	/*
	 * Disable MC68HC901 interrupt to ColdFire core
	 */
	imm->sim.IMR |= MCF5206_SIM_IMR_EINT4;

	isr_remove_handler((void *)ns8390_handler);
}
#endif

/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/

void
mcf5206_init (MCF5206_IMM *imm)
{
	/*
	 * This routine executed upon reset to initialize
	 * MCF5206 periphs.  An initial stack must have been setup
	 * in the SRAM in order for this routine to work.
	 */

	/*
	 * Initialize System Protection Resources.  Software Watchdog
	 * disabled, and Bus Monitor enabled.
	 */
	imm->sim.SYPCR = ( 0
		| MCF5206_SIM_SYPCR_BME		/* yields bus error if no TA */
		| MCF5206_SIM_SYPCR_BM_1024
		) ;

	/*
	 * Disable Bus monitor and SWT while CPU halted.
	 */
	imm->sim.SIMR = ( 0
		| MCF5206_SIM_SIMR_FRZ1
		| MCF5206_SIM_SIMR_FRZ0
		) ;

	/*
	 * Mask all interrupt sources, and ack all.
	 */
	imm->sim.IMR = ( 0
		| MCF5206_SIM_IMR_UART2
		| MCF5206_SIM_IMR_UART1
		| MCF5206_SIM_IMR_MBUS
		| MCF5206_SIM_IMR_T2
		| MCF5206_SIM_IMR_T1
		| MCF5206_SIM_IMR_SWT
		| MCF5206_SIM_IMR_EINT7
		| MCF5206_SIM_IMR_EINT6
		| MCF5206_SIM_IMR_EINT5
		| MCF5206_SIM_IMR_EINT4
		| MCF5206_SIM_IMR_EINT3
		| MCF5206_SIM_IMR_EINT2
		| MCF5206_SIM_IMR_EINT1
		) ;
	imm->sim.IPR = ~0;

	/*
	 * Init PPDDR to all inputs.
	 */
	imm->gpio.PPDDR = 0;

	/*
	 * Initialize PPDAT.
	 */
	imm->gpio.PPDAT = 0;

	/*
	 * Configure multiplexed pins
	 *  - UART2 RTS
	 *  - External interrupts pins are IRQs, not IPLs
	 *  - 8bit Parallel Port I/O, not PST/DDATA
	 *  - WE0, WE1, CS4, CS5
	 */
	imm->sim.PAR = ( 0
		| MCF5206_SIM_PAR_PAR7_RTS2
		| MCF5206_SIM_PAR_PAR1	/* WE0, WE1, CS4, CS5 */
		) ;

	/*
	 * Setup Interrupt Source Vectors
	 */
	imm->sim.ICR1 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* IPL/IRQ 1 */
		| MCF5206_SIM_ICR_IL(1)
		| MCF5206_SIM_ICR_IP(1)
		) ;

	imm->sim.ICR2 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* unusable */
		| MCF5206_SIM_ICR_IL(2)
		| MCF5206_SIM_ICR_IP(3)
		) ;

	imm->sim.ICR3 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* unusable */
		| MCF5206_SIM_ICR_IL(2)
		| MCF5206_SIM_ICR_IP(2)
		) ;

	imm->sim.ICR4 = ( 0
		| MCF5206_SIM_ICR_IL(4)		/* IRQ 4, MC68HC901 */
		| MCF5206_SIM_ICR_IP(2)
		) ;

	imm->sim.ICR5 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* unusable */
		| MCF5206_SIM_ICR_IL(2)
		| MCF5206_SIM_ICR_IP(1)
		) ;

	imm->sim.ICR6 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* unusable */
		| MCF5206_SIM_ICR_IL(2)
		| MCF5206_SIM_ICR_IP(0)
		) ;

	imm->sim.ICR7 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* IPL/IRQ 7, Abort/NMI */
		| MCF5206_SIM_ICR_IL(7)
		| MCF5206_SIM_ICR_IP(3)
		) ;

	imm->sim.ICR8 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* Software Watchdog */
		| MCF5206_SIM_ICR_IL(7)
		| MCF5206_SIM_ICR_IP(2)
		) ;

	imm->sim.ICR9 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* Timer 1 */
		| MCF5206_SIM_ICR_IL(5)
		| MCF5206_SIM_ICR_IP(3)
		) ;

	imm->sim.ICR10 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* Timer 2 */
		| MCF5206_SIM_ICR_IL(5)
		| MCF5206_SIM_ICR_IP(2)
		) ;

	imm->sim.ICR11 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* MBUS */
		| MCF5206_SIM_ICR_IL(3)
		| MCF5206_SIM_ICR_IP(0)
		) ;

	imm->sim.ICR12 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* UART 1 */
		| MCF5206_SIM_ICR_IL(3)
		| MCF5206_SIM_ICR_IP(2)
		) ;

	imm->sim.ICR13 = ( 0
		| MCF5206_SIM_ICR_AVEC		/* UART 2 */
		| MCF5206_SIM_ICR_IL(3)
		| MCF5206_SIM_ICR_IP(1)
		) ;

	/*
	 * Software Watchdog, not used since disabled above
	 */
	imm->sim.SWIVR = 0x0F;			/* uninitiliazed vector */
	imm->sim.SWSR  = 0x55;
	imm->sim.SWSR  = 0xAA;

	/*
	 * Reset timers and disable them
	 */
	imm->timer.TMR1 = ( 0
		| MCF5206_TIMER_TMR_CLK_STOP
		| MCF5206_TIMER_TMR_RST
		) ;

	imm->timer.TER1 = ( 0
		| MCF5206_TIMER_TER_REF
		| MCF5206_TIMER_TER_CAP
		) ;

	imm->timer.TMR2 = ( 0
		| MCF5206_TIMER_TMR_CLK_STOP
		| MCF5206_TIMER_TMR_RST
		) ;

	imm->timer.TER2 = ( 0
		| MCF5206_TIMER_TER_REF
		| MCF5206_TIMER_TER_CAP
		) ;
}

/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/

#define PATTERN_1	(0xA55A9669)
#define PATTERN_2	(0xF0F0F0F0)

static int
probe (NATURAL32 base, NATURAL32 offset)
{
	/*
	 * This routine non-destructively probes a memory address to
	 * determine if physical memory exists, or if the address
	 * is shadowed or non-existant at all.  A return value of 1
	 * indicates physical memory (that is not shadowed) exists at
	 * the address, a return value of 0 indicates that no physical
	 * memory is present.
	 */
	NATURAL32 base_orig;
	NATURAL32 offset_orig;
	NATURAL32 data;

	/*
	 * Step 1) Retain original values of address contents.
	 */
	base_orig = *(NATURAL32 *)(base);
	asm ("	nop");

	offset_orig = *(NATURAL32 *)(base + offset);
	asm ("	nop");

	/*
	 * Step 2) Write Pattern 2 to 'offset'
	 */
	*(NATURAL32 *)(base + offset) = PATTERN_2;
	asm ("	nop");

	/*
	 * Step 3) Pattern 1 'base' in order to detect shadow.
	 */
	*(NATURAL32 *)(base) = PATTERN_1;
	asm ("	nop");

	/*
	 * Step 4) Read 'offset'
	 */
	data = *(NATURAL32 *)(base + offset);
	asm ("	nop");

	/*
	 * Step 5) Restore original values
	 */
	*(NATURAL32 *)(base + offset) = offset_orig;
	asm ("	nop");

	*(NATURAL32 *)(base) = base_orig;
	asm ("	nop");

	if (data == PATTERN_2)
	{
		/*
		 * Real physical memory exists at address
		 */
		return 1;
	}
	else
	{
		/*
		 * No physical memory exists at physical address
		 */
		return 0;
	}
}

/*************************************************************/
void
mcf5206_dramc_init (MCF5206_IMM *imm)
{
	/*
	 * This routine initializes DRAM Controller to setup DRAM.
	 * Setup is for 1M of DRAM in Bank 0.  Actual sizing of
	 * DRAM and check for DRAM Bank 1 is done in board_init().
	 * This is sufficient to run the board.
	 */

	/*
	 * DRAM Controller Refresh Register calculation:
	 *
	 * Refresh cycle time = RC * 16 * ( 1 / System Frequency )
	 *
	 * 15.6us = RC * 16 * ( 1 / 25MHz )
	 *     24 = RC
	 */
	imm->dramc.DCRR = 24;

	imm->dramc.DCTR = 0;

	imm->dramc.DCAR0 = MCF5206_DRAMC_DCAR_BASE(DRAM_ADDRESS);
	imm->dramc.DCMR0 = MCF5206_DRAMC_DCMR_MASK_1M;
	imm->dramc.DCCR0 = ( 0
		| MCF5206_DRAMC_DCCR_PS_32
		| MCF5206_DRAMC_DCCR_BPS_512B
		| MCF5206_DRAMC_DCCR_PM_BURSTPAGE
		| MCF5206_DRAMC_DCCR_WR
		| MCF5206_DRAMC_DCCR_RD
		) ;

	imm->dramc.DCAR1 = 0;
	imm->dramc.DCMR1 = 0;
	imm->dramc.DCCR1 = 0;
}

/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/

void
mcf5206_cs_init (MCF5206_IMM *imm)
{
	/*
	 * This routine initializes ChipSelects to setup peripherals
	 * and other board memory devices.
	 */

	/*
	 * ChipSelect 0 - Boot FLASH 29F010
	 *
	 * ChipSelect 0 is the global chip select coming out of system reset.
	 * CS0 is asserted for every access until CSMR0 is written.  Therefore,
	 * the entire ChipSelect must be properly set prior to asserting
	 * CSCR0_V.
	 */
	imm->cs.CSAR0 = MCF5206_CS_CSAR_BASE(FLASH_ADDRESS);
	imm->cs.CSCR0 = ( 0
		| MCF5206_CS_CSCR_WS(3)
		| MCF5206_CS_CSCR_AA		/* TA_ generated internally */
		| MCF5206_CS_CSCR_PS_16
		| MCF5206_CS_CSCR_WR
		| MCF5206_CS_CSCR_RD
		) ;
	imm->cs.CSMR0 = MCF5206_CS_CSMR_MASK_256K;

	/*
	 * ChipSelect 1 - IACK
	 */
	imm->cs.CSAR1 = MCF5206_IACK_ADDRESS;
	imm->cs.CSMR1 = ( 0
		| MCF5206_IACK_MASK
		| MCF5206_CS_CSMR_SC
		| MCF5206_CS_CSMR_SD
		| MCF5206_CS_CSMR_UC
		| MCF5206_CS_CSMR_UD
		) ;
	imm->cs.CSCR1 = ( 0
		| MCF5206_CS_CSCR_PS_8
		| MCF5206_CS_CSCR_WR
		| MCF5206_CS_CSCR_RD
		) ;

	/*
	 * ChipSelect 2 - MC68HC901
	 */
	imm->cs.CSAR2 = MCF5206_CS_CSAR_BASE(MC68HC901_ADDRESS);
	imm->cs.CSMR2 = ( 0
		| MCF5206_CS_CSMR_MASK_64K
		| MCF5206_CS_CSMR_SC
		| MCF5206_CS_CSMR_UC
		) ;
	imm->cs.CSCR2 = ( 0
		| MCF5206_CS_CSCR_PS_8
		| MCF5206_CS_CSCR_WR
		| MCF5206_CS_CSCR_RD
		) ;

	/*
	 * ChipSelect 3 - ISA FPGA - Externally terminated
	 */
	imm->cs.CSAR3 = MCF5206_CS_CSAR_BASE(ISA_ADDRESS);
	imm->cs.CSMR3 = ( 0
		| MCF5206_CS_CSMR_MASK_1M
		) ;
	imm->cs.CSCR3 = ( 0
		| MCF5206_CS_CSCR_PS_16
		| MCF5206_CS_CSCR_WR
		| MCF5206_CS_CSCR_RD
		) ;

	/*
	 * ChipSelect 4,5,6 and 7 - Empty
	 */
	imm->cs.CSAR4 = 0;
	imm->cs.CSMR4 = 0;
	imm->cs.CSCR4 = 0;
	imm->cs.CSAR5 = 0;
	imm->cs.CSMR5 = 0;
	imm->cs.CSCR5 = 0;
	imm->cs.CSAR6 = 0;
	imm->cs.CSMR6 = 0;
	imm->cs.CSCR6 = 0;
	imm->cs.CSAR7 = 0;
	imm->cs.CSMR7 = 0;
	imm->cs.CSCR7 = 0;

	/*
	 * Default Memory Configuration Register, DMCR.  Since Bus Monitor
	 * enabled above, and DMCR ignores default memory, we should get
	 * a bus error on accessing default memory.
	 */
	imm->cs.DMCR = 0;
}

/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/

/****************************************************************/
void
board_set_nicbase (int argc, char **argv)
{
	/*
	 * This routine is called by set/show to set or display the
	 * base address of the NE2000 card.
	 */
	int success, addr;
	PARAM *param;

	if (argc == 2)
	{
		/*
		 * Display NE2000 base address
		 */
		param = (PARAM *)&board.flash->param;
		printf("%#03X\n",param->nicbase);
	}
	else
	{
		addr = get_value(argv[2], &success, 16);
		if (success)
		{
			param = (PARAM *)__USER_SPACE;
			addr = (addr & 0x00000FE0);
		    update_board_params((BYTE *)&addr, 4,
		        (NATURAL32)((NATURAL32)&param->nicbase -
				(NATURAL32)__USER_SPACE));
		}
		else
		{
			printf("Error: Invalid base address: %s\n",argv[2]);
		}
	}
}

/****************************************************************/
NATURAL32
board_get_nicbase (void)
{
	PARAM *param;
	param = (PARAM *)&board.flash->param;

	return param->nicbase;
}

/****************************************************************/
void
board_set_nicirq (int argc, char **argv)
{
	/*
	 * This routine is called by set/show to set or display the
	 * IRQ of the NE2000 card.
	 */
	int success, irq;
	PARAM *param;

	if (argc == 2)
	{
		/*
		 * Display NE2000 IRQ
		 */
		param = (PARAM *)&board.flash->param;
		printf("%d\n",param->nicirq);
	}
	else
	{
		irq = get_value(argv[2], &success, 10);
		if (success)
		{
			switch (irq)
			{
				case 3:
				case 4:
				case 5:
				case 6:
				case 7:
				case 9:
				case 10:
				case 11:
					param = (PARAM *)__USER_SPACE;
				    update_board_params((BYTE *)&irq, 4,
				        (NATURAL32)((NATURAL32)&param->nicirq -
						(NATURAL32)__USER_SPACE));
					break;
				default:
					printf("Error: Invalid IRQ: %s\n",argv[2]);
					break;
			}
		}
		else
		{
			printf("Error: Invalid IRQ: %s\n",argv[2]);
		}
	}
}

/****************************************************************/
NATURAL32
board_get_nicirq (void)
{
	PARAM *param;
	param = (PARAM *)&board.flash->param;

	return param->nicirq;
}

/****************************************************************/
