/*
 * File:		cpu.h
 * Purpose:		Prototypes and definitions for generic CPU functions.
 *
 * Notes:
 *
 * Date:		3-13-96
 *
 * Modifications:
 *
 */

/*
 * This includes CPU specific information as well as provides function
 * prototypes and additional commands to dBUG.
 */
#include "cpus.h"

#ifndef _CPU_H
#define _CPU_H

typedef NATURAL8			uint8;
typedef NATURAL8			nat8;
typedef NATURAL8			UINT8;
typedef NATURAL8			NAT8;
typedef NATURAL16			uint16;
typedef NATURAL16			nat16;
typedef NATURAL16			UINT16;
typedef NATURAL16			NAT16;
typedef NATURAL32			uint32;
typedef NATURAL32			nat32;
typedef NATURAL32			UINT32;
typedef NATURAL32			NAT32;
typedef INTEGER8			int8;
typedef INTEGER8			INT8;
typedef INTEGER16			int16;
typedef INTEGER16			INT16;
typedef INTEGER32			int32;
typedef INTEGER32			INT32;
typedef volatile NATURAL8	reg8;
typedef volatile NATURAL8	REG8;
typedef volatile NATURAL16	reg16;
typedef volatile NATURAL16	REG16;
typedef volatile NATURAL32	reg32;
typedef volatile NATURAL32	REG32;



typedef struct
{
	char	*reg_name;		/* name user will type in to reference it	*/
	void	*reg_offset;	/* pointer into REGISTERS data struct		*/
	int		reg_size;		/* size of register in bits					*/
} reg_entry;
 

/*
 * Prototypes
 */
void
asm_switch_context (void *);

void
cpu_run_user_code (int);

void
cpu_set_register (char *, NATURAL32);

void
cpu_set_program_counter (ADDRESS);

ADDRESS
cpu_get_program_counter (void);

int
cpu_parse_size (char *);

int
cpu_invalid_instr_addr (ADDRESS);

void
cpu_display_register(char *);

void
cpu_init (void);

void
cpu_write_data (ADDRESS, int, int);

int
cpu_read_data (ADDRESS, int);

ADDRESS
cpu_align_address (ADDRESS, int);

void
cpu_icache_flush (void);

void
cpu_dcache_flush (void);

ADDRESS
cpu_disasm (ADDRESS, int);

void
asm_startmeup (void);

uint32
cpu_ior_8 (void *);

uint32
cpu_ior_16 (void *);

uint32
cpu_ior_32 (void *);

void
cpu_iow_8 (void *, uint32);

void
cpu_iow_16 (void *, uint32);

void
cpu_iow_32 (void *, uint32);



#endif	/* _CPU_H */
