/*
 * File:		mc68k_types.h
 * Purpose:		Basic CPU data type definitions.
 *
 * Notes:
 *
 * Date:		12-18-95
 *
 * Modifications:
 *
 */

#ifndef _MC68K_TYPES_H
#define _MC68K_TYPES_H

typedef unsigned char		BYTE;	/*  8 bits */
typedef unsigned short int	WORD;	/* 16 bits */
typedef unsigned long int	LONG;	/* 32 bits */

typedef signed char			SBYTE;	/*  8 bits */
typedef signed short int	SWORD;	/* 16 bits */
typedef signed long int		SLONG;	/* 32 bits */

typedef unsigned char		NATURAL8;  /*  8 bits */
typedef unsigned short int	NATURAL16; /* 16 bits */
typedef unsigned long int	NATURAL32; /* 32 bits */

typedef signed char			INTEGER8;   /*  8 bits */
typedef signed short int	INTEGER16;  /* 16 bits */
typedef signed long int		INTEGER32;  /* 32 bits */

typedef NATURAL32 ADDRESS;

/*
 * The following defitions are primarily for peripherals which are
 * not connected completely to the data bus.
 */

/*
 * This definition defines a peripheral which is 8-bit wide connected
 * to the least signficant byte-lane of the 32-bit data bus.
 */
typedef struct
{
	BYTE _a, _b, _c, data;
} B32LSB;

/*
 * This definition defines a peripheral which is 8-bit wide connected
 * to the most signficant byte-lane of the 32-bit data bus.
 */
typedef struct
{
	BYTE data, _a, _b, _c;
} B32MSB;

/*
 * This definition defines a peripheral which is 16-bits wide connected
 * to the least signficant 16-bits of the 32-bit data bus.
 */
typedef struct
{
	WORD _a, data;
} W32LSB;

/*
 * This definition defines a peripheral which is 16-bits wide connected
 * to the most signficant 16-bits of the 32-bit data bus.
 */
typedef struct
{
	WORD data, _a;
} W32MSB;

/*
 * This definition defines a peripheral which is 8-bit wide connected
 * to the least signficant byte-lane of the 16-bit data bus.
 */
typedef struct
{
	BYTE _a, data;
} B16LSB;

/*
 * This definition defines a peripheral which is 8-bit wide connected
 * to the most signficant byte-lane of the 16-bit data bus.
 */
typedef struct
{
	BYTE data, _a;
} B16MSB;


#endif	/* _MC68K_TYPES_H */
