;
; File:		mcf5206_lo.s
; Purpose:	Lowest level routines for MCF5206, including context
;			switching and exception handling.
;
; Notes:
;
; Date:		10-3-96
;
; Modifications:
;
;

	; This file contains the low level assembly routines to do
	; things that C can not.  It also contains the two initial
	; vectors needed for ROM boot-up.

	xdef	_asm_switch_context
	xdef	_asm_exception_handler
	xdef	_asm_set_ipl
	xdef	_dbug_sc_handler
	xdef	_asm_sc_exit_to_dbug
	xdef	_cpu_isr_handler
	xdef	_cpu_icache_flush
	xdef	_cpu_dcache_flush
	xdef	_asm_startmeup
	xdef	_mcf5206_write_cacr
	xdef	_mcf5206_write_acr0
	xdef	_mcf5206_write_acr1
	xdef	_mcf5206_write_vbr
	xdef	_mcf5206_write_srambar
	xdef	_mcf5206_write_mbar

	xdef	_VECTOR_TABLE


	.text

;
; Exception Vector Table
;
_VECTOR_TABLE:
INITSP:		dc.l	___SP_INIT				; Initial SP
INITPC:		dc.l	_asm_startmeup			; Initial PC
vector02:	dc.l	_asm_exception_handler	; Access Error
vector03:	dc.l	_asm_exception_handler	; Address Error
vector04:	dc.l	_asm_exception_handler	; Illegal Instruction
vector05:	dc.l	_asm_exception_handler	; Reserved
vector06:	dc.l	_asm_exception_handler	; Reserved
vector07:	dc.l	_asm_exception_handler	; Reserved
vector08:	dc.l	_asm_exception_handler	; Privilege Violation
vector09:	dc.l	_asm_exception_handler	; Trace
vector0A:	dc.l	_asm_exception_handler	; Unimplemented A-Line
vector0B:	dc.l	_asm_exception_handler	; Unimplemented F-Line
vector0C:	dc.l	_asm_exception_handler	; Debug Interrupt
vector0D:	dc.l	_asm_exception_handler	; Reserved
vector0E:	dc.l	_asm_exception_handler	; Format Error
vector0F:	dc.l	_asm_exception_handler	; Unitialized Interrupt
vector10:	dc.l	_asm_exception_handler	; Reserved
vector11:	dc.l	_asm_exception_handler	; Reserved
vector12:	dc.l	_asm_exception_handler	; Reserved
vector13:	dc.l	_asm_exception_handler	; Reserved
vector14:	dc.l	_asm_exception_handler	; Reserved
vector15:	dc.l	_asm_exception_handler	; Reserved
vector16:	dc.l	_asm_exception_handler	; Reserved
vector17:	dc.l	_asm_exception_handler	; Reserved
vector18:	dc.l	_asm_exception_handler	; Spurious Interrupt
vector19:	dc.l	_cpu_isr_handler		; Autovector Level 1
vector1A:	dc.l	_cpu_isr_handler		; Autovector Level 2
vector1B:	dc.l	_cpu_isr_handler		; Autovector Level 3
vector1C:	dc.l	_cpu_isr_handler		; Autovector Level 4
vector1D:	dc.l	_cpu_isr_handler		; Autovector Level 5
vector1E:	dc.l	_cpu_isr_handler		; Autovector Level 6
vector1F:	dc.l	_cpu_isr_handler		; Autovector Level 7
vector20:	dc.l	_asm_exception_handler	; TRAP #0
vector21:	dc.l	_asm_exception_handler	; TRAP #1
vector22:	dc.l	_asm_exception_handler	; TRAP #2
vector23:	dc.l	_asm_exception_handler	; TRAP #3
vector24:	dc.l	_asm_exception_handler	; TRAP #4
vector25:	dc.l	_asm_exception_handler	; TRAP #5
vector26:	dc.l	_asm_exception_handler	; TRAP #6
vector27:	dc.l	_asm_exception_handler	; TRAP #7
vector28:	dc.l	_asm_exception_handler	; TRAP #8
vector29:	dc.l	_asm_exception_handler	; TRAP #9
vector2A:	dc.l	_asm_exception_handler	; TRAP #10
vector2B:	dc.l	_asm_exception_handler	; TRAP #11
vector2C:	dc.l	_asm_exception_handler	; TRAP #12
vector2D:	dc.l	_asm_exception_handler	; TRAP #13
vector2E:	dc.l	_asm_exception_handler	; TRAP #14
vector2F:	dc.l	_dbug_sc_handler		; TRAP #15 - System Call
vector30:	dc.l	_asm_exception_handler	; Reserved
vector31:	dc.l	_asm_exception_handler	; Reserved
vector32:	dc.l	_asm_exception_handler	; Reserved
vector33:	dc.l	_asm_exception_handler	; Reserved
vector34:	dc.l	_asm_exception_handler	; Reserved
vector35:	dc.l	_asm_exception_handler	; Reserved
vector36:	dc.l	_asm_exception_handler	; Reserved
vector37:	dc.l	_asm_exception_handler	; Reserved
vector38:	dc.l	_asm_exception_handler	; Reserved
vector39:	dc.l	_asm_exception_handler	; Reserved
vector3A:	dc.l	_asm_exception_handler	; Reserved
vector3B:	dc.l	_asm_exception_handler	; Reserved
vector3C:	dc.l	_asm_exception_handler	; Reserved
vector3D:	dc.l	_asm_exception_handler	; Reserved
vector3E:	dc.l	_asm_exception_handler	; Reserved
vector3F:	dc.l	_asm_exception_handler	; Reserved
vector40:	dc.l	_cpu_isr_handler		; User Defined Interrupts
vector41:	dc.l	_cpu_isr_handler
vector42:	dc.l	_cpu_isr_handler
vector43:	dc.l	_cpu_isr_handler
vector44:	dc.l	_cpu_isr_handler
vector45:	dc.l	_cpu_isr_handler
vector46:	dc.l	_cpu_isr_handler
vector47:	dc.l	_cpu_isr_handler
vector48:	dc.l	_cpu_isr_handler
vector49:	dc.l	_cpu_isr_handler
vector4A:	dc.l	_cpu_isr_handler
vector4B:	dc.l	_cpu_isr_handler
vector4C:	dc.l	_cpu_isr_handler
vector4D:	dc.l	_cpu_isr_handler
vector4E:	dc.l	_cpu_isr_handler
vector4F:	dc.l	_cpu_isr_handler
vector50:	dc.l	_cpu_isr_handler
vector51:	dc.l	_cpu_isr_handler
vector52:	dc.l	_cpu_isr_handler
vector53:	dc.l	_cpu_isr_handler
vector54:	dc.l	_cpu_isr_handler
vector55:	dc.l	_cpu_isr_handler
vector56:	dc.l	_cpu_isr_handler
vector57:	dc.l	_cpu_isr_handler
vector58:	dc.l	_cpu_isr_handler
vector59:	dc.l	_cpu_isr_handler
vector5A:	dc.l	_cpu_isr_handler
vector5B:	dc.l	_cpu_isr_handler
vector5C:	dc.l	_cpu_isr_handler
vector5D:	dc.l	_cpu_isr_handler
vector5E:	dc.l	_cpu_isr_handler
vector5F:	dc.l	_cpu_isr_handler
vector60:	dc.l	_cpu_isr_handler
vector61:	dc.l	_cpu_isr_handler
vector62:	dc.l	_cpu_isr_handler
vector63:	dc.l	_cpu_isr_handler
vector64:	dc.l	_cpu_isr_handler
vector65:	dc.l	_cpu_isr_handler
vector66:	dc.l	_cpu_isr_handler
vector67:	dc.l	_cpu_isr_handler
vector68:	dc.l	_cpu_isr_handler
vector69:	dc.l	_cpu_isr_handler
vector6A:	dc.l	_cpu_isr_handler
vector6B:	dc.l	_cpu_isr_handler
vector6C:	dc.l	_cpu_isr_handler
vector6D:	dc.l	_cpu_isr_handler
vector6E:	dc.l	_cpu_isr_handler
vector6F:	dc.l	_cpu_isr_handler
vector70:	dc.l	_cpu_isr_handler
vector71:	dc.l	_cpu_isr_handler
vector72:	dc.l	_cpu_isr_handler
vector73:	dc.l	_cpu_isr_handler
vector74:	dc.l	_cpu_isr_handler
vector75:	dc.l	_cpu_isr_handler
vector76:	dc.l	_cpu_isr_handler
vector77:	dc.l	_cpu_isr_handler
vector78:	dc.l	_cpu_isr_handler
vector79:	dc.l	_cpu_isr_handler
vector7A:	dc.l	_cpu_isr_handler
vector7B:	dc.l	_cpu_isr_handler
vector7C:	dc.l	_cpu_isr_handler
vector7D:	dc.l	_cpu_isr_handler
vector7E:	dc.l	_cpu_isr_handler
vector7F:	dc.l	_cpu_isr_handler
vector80:	dc.l	_cpu_isr_handler
vector81:	dc.l	_cpu_isr_handler
vector82:	dc.l	_cpu_isr_handler
vector83:	dc.l	_cpu_isr_handler
vector84:	dc.l	_cpu_isr_handler
vector85:	dc.l	_cpu_isr_handler
vector86:	dc.l	_cpu_isr_handler
vector87:	dc.l	_cpu_isr_handler
vector88:	dc.l	_cpu_isr_handler
vector89:	dc.l	_cpu_isr_handler
vector8A:	dc.l	_cpu_isr_handler
vector8B:	dc.l	_cpu_isr_handler
vector8C:	dc.l	_cpu_isr_handler
vector8D:	dc.l	_cpu_isr_handler
vector8E:	dc.l	_cpu_isr_handler
vector8F:	dc.l	_cpu_isr_handler
vector90:	dc.l	_cpu_isr_handler
vector91:	dc.l	_cpu_isr_handler
vector92:	dc.l	_cpu_isr_handler
vector93:	dc.l	_cpu_isr_handler
vector94:	dc.l	_cpu_isr_handler
vector95:	dc.l	_cpu_isr_handler
vector96:	dc.l	_cpu_isr_handler
vector97:	dc.l	_cpu_isr_handler
vector98:	dc.l	_cpu_isr_handler
vector99:	dc.l	_cpu_isr_handler
vector9A:	dc.l	_cpu_isr_handler
vector9B:	dc.l	_cpu_isr_handler
vector9C:	dc.l	_cpu_isr_handler
vector9D:	dc.l	_cpu_isr_handler
vector9E:	dc.l	_cpu_isr_handler
vector9F:	dc.l	_cpu_isr_handler
vectorA0:	dc.l	_cpu_isr_handler
vectorA1:	dc.l	_cpu_isr_handler
vectorA2:	dc.l	_cpu_isr_handler
vectorA3:	dc.l	_cpu_isr_handler
vectorA4:	dc.l	_cpu_isr_handler
vectorA5:	dc.l	_cpu_isr_handler
vectorA6:	dc.l	_cpu_isr_handler
vectorA7:	dc.l	_cpu_isr_handler
vectorA8:	dc.l	_cpu_isr_handler
vectorA9:	dc.l	_cpu_isr_handler
vectorAA:	dc.l	_cpu_isr_handler
vectorAB:	dc.l	_cpu_isr_handler
vectorAC:	dc.l	_cpu_isr_handler
vectorAD:	dc.l	_cpu_isr_handler
vectorAE:	dc.l	_cpu_isr_handler
vectorAF:	dc.l	_cpu_isr_handler
vectorB0:	dc.l	_cpu_isr_handler
vectorB1:	dc.l	_cpu_isr_handler
vectorB2:	dc.l	_cpu_isr_handler
vectorB3:	dc.l	_cpu_isr_handler
vectorB4:	dc.l	_cpu_isr_handler
vectorB5:	dc.l	_cpu_isr_handler
vectorB6:	dc.l	_cpu_isr_handler
vectorB7:	dc.l	_cpu_isr_handler
vectorB8:	dc.l	_cpu_isr_handler
vectorB9:	dc.l	_cpu_isr_handler
vectorBA:	dc.l	_cpu_isr_handler
vectorBB:	dc.l	_cpu_isr_handler
vectorBC:	dc.l	_cpu_isr_handler
vectorBD:	dc.l	_cpu_isr_handler
vectorBE:	dc.l	_cpu_isr_handler
vectorBF:	dc.l	_cpu_isr_handler
vectorC0:	dc.l	_cpu_isr_handler
vectorC1:	dc.l	_cpu_isr_handler
vectorC2:	dc.l	_cpu_isr_handler
vectorC3:	dc.l	_cpu_isr_handler
vectorC4:	dc.l	_cpu_isr_handler
vectorC5:	dc.l	_cpu_isr_handler
vectorC6:	dc.l	_cpu_isr_handler
vectorC7:	dc.l	_cpu_isr_handler
vectorC8:	dc.l	_cpu_isr_handler
vectorC9:	dc.l	_cpu_isr_handler
vectorCA:	dc.l	_cpu_isr_handler
vectorCB:	dc.l	_cpu_isr_handler
vectorCC:	dc.l	_cpu_isr_handler
vectorCD:	dc.l	_cpu_isr_handler
vectorCE:	dc.l	_cpu_isr_handler
vectorCF:	dc.l	_cpu_isr_handler
vectorD0:	dc.l	_cpu_isr_handler
vectorD1:	dc.l	_cpu_isr_handler
vectorD2:	dc.l	_cpu_isr_handler
vectorD3:	dc.l	_cpu_isr_handler
vectorD4:	dc.l	_cpu_isr_handler
vectorD5:	dc.l	_cpu_isr_handler
vectorD6:	dc.l	_cpu_isr_handler
vectorD7:	dc.l	_cpu_isr_handler
vectorD8:	dc.l	_cpu_isr_handler
vectorD9:	dc.l	_cpu_isr_handler
vectorDA:	dc.l	_cpu_isr_handler
vectorDB:	dc.l	_cpu_isr_handler
vectorDC:	dc.l	_cpu_isr_handler
vectorDD:	dc.l	_cpu_isr_handler
vectorDE:	dc.l	_cpu_isr_handler
vectorDF:	dc.l	_cpu_isr_handler
vectorE0:	dc.l	_cpu_isr_handler
vectorE1:	dc.l	_cpu_isr_handler
vectorE2:	dc.l	_cpu_isr_handler
vectorE3:	dc.l	_cpu_isr_handler
vectorE4:	dc.l	_cpu_isr_handler
vectorE5:	dc.l	_cpu_isr_handler
vectorE6:	dc.l	_cpu_isr_handler
vectorE7:	dc.l	_cpu_isr_handler
vectorE8:	dc.l	_cpu_isr_handler
vectorE9:	dc.l	_cpu_isr_handler
vectorEA:	dc.l	_cpu_isr_handler
vectorEB:	dc.l	_cpu_isr_handler
vectorEC:	dc.l	_cpu_isr_handler
vectorED:	dc.l	_cpu_isr_handler
vectorEE:	dc.l	_cpu_isr_handler
vectorEF:	dc.l	_cpu_isr_handler
vectorF0:	dc.l	_cpu_isr_handler
vectorF1:	dc.l	_cpu_isr_handler
vectorF2:	dc.l	_cpu_isr_handler
vectorF3:	dc.l	_cpu_isr_handler
vectorF4:	dc.l	_cpu_isr_handler
vectorF5:	dc.l	_cpu_isr_handler
vectorF6:	dc.l	_cpu_isr_handler
vectorF7:	dc.l	_cpu_isr_handler
vectorF8:	dc.l	_cpu_isr_handler
vectorF9:	dc.l	_cpu_isr_handler
vectorFA:	dc.l	_cpu_isr_handler
vectorFB:	dc.l	_cpu_isr_handler
vectorFC:	dc.l	_cpu_isr_handler
vectorFD:	dc.l	_cpu_isr_handler
vectorFE:	dc.l	_cpu_isr_handler
vectorFF:	dc.l	_cpu_isr_handler

	.align 4


	; The following defines the registers that are
	; maintained by the debugger.  The definition below *MUST*
	; match that in `mcf5206.h'.

r_d0:	equ		0
r_d1:	equ		4
r_d2:	equ		8
r_d3:	equ		12
r_d4:	equ		16
r_d5:	equ		20
r_d6:	equ		24
r_d7:	equ		28
r_a0:	equ		32
r_a1:	equ		36
r_a2:	equ		40
r_a3:	equ		44
r_a4:	equ		48
r_a5:	equ		52
r_a6:	equ		56
r_a7:	equ		60
r_pc:	equ		64
r_sr:	equ		68
		
r_sp:	equ		r_a7	; on ColdFire, A7 is SP

IMMaddr:	equ	0xF0000000
SRAMaddr:	equ	0xE0000000
SRAMsize:	equ	512

	section	.text

;
; This is the main entry point upon hard reset.
;
_asm_startmeup:
	move.w		#0x2700,SR

	; As long as the boot ROM is aligned to a 1M boundary, you can do
	; the following.  If it is not, then use the code that places
	; VBR at 0x00000000.
	; 
	move.l      #_VECTOR_TABLE,d0
	; moveq.l   #0,d0

	; Bug in MCF52xx.  VBR only updated from A7
	;
	move.l		d0,a7
	movec       a7,VBR

	; Invalidate the cache and disable it
	move.l	#0x01000000,d0
	dc.l	0x4e7b0002		; movec d0,cacr

	; Disable ACRs
	moveq.l	#0,d0
	dc.l	0x4e7b0004		; movec d0,ACR0
	dc.l	0x4e7b0005		; movec d0,ACR1

	; Initialize SRAMBAR
	move.l	#SRAMaddr+0x00000001,d0	; locate SRAM, validate it!
	dc.l	0x4e7b0C04				; movec d0,SRAMBAR

	; Initialize MBAR
	move.l	#IMMaddr+0x00000001,d0	; locate MBAR, validate it!
	dc.l	0x4e7b0C0F				; movec d0,MBAR

	nop
	nop

	; Point SP into SRAM (temporarily).  SRAM is used as stack space
	; while initializing the MCF5206 periphs and memory controller.
	;
	move.l	#SRAMaddr+SRAMsize,SP

	; Initialize MCF5206 periphs, etc
	move.l	#IMMaddr,-(sp)	; pointer to internal resources
	jsr		_mcf5206_init
	lea		4(sp),sp

	; Setup DRAM Controller
	move.l	#IMMaddr,-(sp)	; pointer to internal resources
	jsr		_mcf5206_dramc_init
	lea		4(sp),sp

	; Setup Chipselects
	move.l	#IMMaddr,-(sp)	; pointer to internal resources
	jsr		_mcf5206_cs_init
	lea		4(sp),sp

	; Point SP to DRAM.  DRAM may only be visible once CS0 
	; de-activated as global chipselect.
	move.l	#___SP_INIT,sp

	nop
	nop

	jmp			_main



;
; This routine is responsible for restoring a user
; register set and executing it.
;
; call from C:  asm_switch_context(REGISTERS *context);
;
_asm_switch_context:

    move.l      4(sp),a0        ; point to cpu data structure

	; ColdFire 5200 requires stack frames be 0 modula 4

	; calculate format field encoding
	move.l	r_sp(a0),d6			; sp for stack adjustment
	move.l	d6,d7				; sp for comparisons
	andi.l	#$00000003,d7		; last two bits of a7
	beq.s	ff0100				; format field 0100
	cmpi.l	#1,d7
	beq.s	ff0101				; format field 0101
	cmpi.l	#2,d7
	beq.s	ff0110				; format field 0110
	; must be format field 0111

ff0111:
	subq.l	#3,d6			; adjust sp 3 bytes to align
	move.l	#$70000000,d7	; encode format 0111
	bra.s	write_frame
ff0110:
	subq.l	#2,d6			; adjust sp 2 bytes to align
	move.l	#$60000000,d7	; encode format 0110
	bra.s	write_frame
ff0101:
	subq.l	#1,d6			; adjust sp 1 bytes to align
	move.l	#$50000000,d7	; encode format 0101
	bra.s	write_frame
ff0100:
	; no stack adjustment necessary
	move.l	#$40000000,d7	; encode format 0100
		

write_frame:

;	move.l	d6,sp			; sp (change sp to user stack)
;	move.l	r_pc(a0),-(sp)	; pc
;	move.w	r_sr(a0),d7		; sr (format already in d7)
;	move.l	d7,-(sp)

    ; Build stack frame before switching stacks
    move.l  d6,a6
    move.l  r_pc(a0),-(a6)  ; sp
    move.w  r_sr(a0),d7     ; sr (format already in d7)
    move.l  d7,-(a6)
    nop                     ; force pending writes
    move.l  a6,sp
 
    ; the PC and SR are in the stack frame
    ; movem.l (a0),d0-d7/a0-a6    ; remaining registers
    ; or could use ...
    move.l  r_d7(a0),d7
    move.l  r_d6(a0),d6
    move.l  r_d5(a0),d5
    move.l  r_d4(a0),d4
    move.l  r_d3(a0),d3
    move.l  r_d2(a0),d2
    move.l  r_d1(a0),d1
    move.l  r_d0(a0),d0
    move.l  r_a6(a0),a6
    move.l  r_a5(a0),a5
    move.l  r_a4(a0),a4
    move.l  r_a3(a0),a3
    move.l  r_a2(a0),a2
    move.l  r_a1(a0),a1
    move.l  r_a0(a0),a0

	; here we go!
	nop
	rte


	; This routine is the lowest-level exception handler for ColdFire.
	; This routine switches SP to a known-good RAM location, and then
	; saves the registers.  This routine should work even if the user
	; code has a bad stack pointer.
_asm_exception_handler:

	move.w	#0x2700,sr	; if doing interrupt i/o, then comment out

	; Change to dBUG stack space
	move.l	sp,___SP_INIT
	move.l	#___SP_INIT,sp

	move.l	a0,-(sp)
	lea		_context,a0
	movem.l	d0-d7/a0-a7,(a0); d0-d7/a0-a7 (a0/a7 are bad)
	move.l	(sp)+,r_a0(a0)	; a0
	move.l	(sp),a1
	move.l	a1,r_sp(a0)		; sp/a7 at end of stack frame
	move.l	4(a1),r_pc(a0)	; pc
	move.w	2(a1),r_sr(a0)	; sr

	; all regs are saved and correct except SP, but SP taken care of
	; in cpu_handler():  int cpu_handler (STACK_FRAME *framep);
	move.l	a1,-(sp)	; arg for cpu_handler()

	; Flush cache and disable it.
	jsr		_cpu_icache_flush

	jsr		_cpu_handler
	lea		4(sp),sp
	tst.l	d0
	bne.s	backtomonitor

	; restore registers modified:  a0,d0,a1,d1,sp
	; sr in stack frame
	lea		_context,a0

	; sp must have been adjusted properly for RTE
	; since cpu_handler() initially knocks off the
	; frame from the user stack. (see ispdelta)
	move.l	r_sp(a0),sp	;

	move.l	r_d1(a0),d1	;
	move.l	r_a1(a0),a1	;
	move.l	r_d0(a0),d0	;
	move.l	r_a0(a0),a0	; This one must be done LAST!!!

	rte

backtomonitor:
	move.w	#0x2700,sr	; if not doing interrupt i/o
	;move.w	#0x2000,sr	; if doing interrupt driven i/o
	jmp	_mainloop



;
; This routine is called from within/by a user system call to save the
; context and return to dBUG cleanly.  This routine expects a pointer
; to the exception stack frame.  This pointer provides three pieces of
; information.  1) The SP (which points to end of exception stack frame),
; 2) the PC, and 3) the SR.  All other regs are preserved from their
; values entering this routine.  This routine must be called via a JSR
; or BSR instruction.
;
; So, stack coming into this routine looks like:
;
;                  |       ...        |
;                  |    Return PC     |
;                  | Format/Vector/SR | <-- SP after exception
;                  |       ...        |
;                  |  Stack Frame Ptr | (The SP after exception)
;                  |   JSR Return PC  | (not used)
;                  |                  |
;
_asm_sc_exit_to_dbug:
		; Save off general purpose registers
		;
		move.w		#0x2700,sr

		move.l		a0,-(sp)
		lea			_context,a0
		move.l		d0,r_d0(a0)
		move.l		d1,r_d1(a0)
		move.l		d2,r_d2(a0)
		move.l		d3,r_d3(a0)
		move.l		d4,r_d4(a0)
		move.l		d5,r_d5(a0)
		move.l		d6,r_d6(a0)
		move.l		d7,r_d7(a0)
		move.l		a1,r_a1(a0)
		move.l		a2,r_a2(a0)
		move.l		a3,r_a3(a0)
		move.l		a4,r_a4(a0)
		move.l		a5,r_a5(a0)
		move.l		a6,r_a6(a0)
		move.l		(sp)+,d0
		move.l		d0,r_a0(a0)

		; Now get info from stack frame pointer
		;
		move.l		4(sp),a1		; Stack frame pointer argument
		move.l		a1,r_a7(a0) 	; SP at end of exception
		move.l		4(a1),r_pc(a0)	; pc
		move.w		2(a1),r_sr(a0)	; sr

		; Now that context has been stored off, change to dBUG stack
		; and prepare to enter the debugger again.
		;
		move.l		#___SP_INIT,sp

		jsr			_cpu_icache_flush
		
		jmp			_mainloop
		


; This routines changes the IPL to the value passed into the routine.
; It also returns the old IPL value back.
; Calling convention from C:
;   old_ipl = asm_set_ipl(new_ipl);
; For the Diab Data C compiler, it passes return value thru D0.
; Note that only the least significant three bits of the passed
; value are used.
	section .text
_asm_set_ipl:
	link	a6,#-8
	movem.l	d6-d7,(sp)

		move.w	sr,d7		; current sr

		move.l	d7,d0		; prepare return value
		andi.l	#$0700,d0	; mask out IPL
		lsr.l	#8,d0		; IPL

		move.l	8(a6),d6	; get argument
		andi.l	#$07,d6		; least significant three bits
		lsl.l	#8,d6		; move over to make mask

		andi.l	#$0000F8FF,d7	; zero out current IPL
		or.l	d6,d7			; place new IPL in sr
		move.w	d7,sr

	movem.l	(sp),d6-d7
	lea		8(sp),sp
	unlk	a6
	rts

;
; Cache flushing routines for the MCF5206.  MCF5206 has ICACHE only.
; ICACHE is invalidated and disabled.
;
_cpu_icache_flush:
	nop						; sync
	move.l	#0x01000000,d0
	dc.l	0x4e7b0002		; movec d0,cacr
	nop						; sync
	dc.l	0x4e7b0002		; movec d0,cacr
	rts

_cpu_dcache_flush:
	rts

;
; This is the low-level Interrupt Service Routine code
;
_cpu_isr_handler:
	link		a6,#-16
	movem.l		d0-d1/a0-a1,(sp)

	move.w		4(a6),d0
	lsr.l		#2,d0
	andi.l		#0x0000FF,d0
	move.l		d0,-(sp)
	jsr			_isr_execute_handler
	lea			4(sp),sp
	cmpi.l		#1,d0
	beq			handled

nothandled:
	movem.l		(sp),d0-d1/a0-a1
	unlk		a6
	jmp			_asm_exception_handler

handled:
	movem.l		(sp),d0-d1/a0-a1
	unlk		a6
	rte

;
; These routines write to the special purpose registers in the ColdFire
; core.  Since these registers are write-only in the supervisor model,
; no corresponding read routines exist.
;
_mcf5206_write_cacr:
	move.l	4(sp),d0
	dc.l	0x4e7b0002		; movec d0,cacr
	nop
	rts

_mcf5206_write_acr0:
	move.l	4(sp),d0
	dc.l	0x4e7b0004		; movec d0,ACR0
	nop
	rts

_mcf5206_write_acr1:
	move.l	4(sp),d0
	dc.l	0x4e7b0005		; movec d0,ACR1
	nop
	rts

_mcf5206_write_vbr:
	move.l	4(sp),d0
	andi.l	#0xFFF00000,d0	; align to 1M boundary
	move.l	sp,d1
	move.l	d0,sp
	movec	a7,VBR
	move.l	d1,sp
	nop
	rts

_mcf5206_write_srambar:
	move.l	4(sp),d0
	dc.l	0x4e7b0C04		; movec d0,SRAMBAR
	nop
	rts

_mcf5206_write_mbar:
	move.l	4(sp),d0
	dc.l	0x4e7b0C0F		; movec d0,MBAR
	nop
	rts


	end
