/*
 * File:		ns8390.h
 * Purpose:		Header file for the National Semiconductor NS8390
 *				Ethernet chipset, commonly used in NE2000-type cards.
 *
 * Notes:
 *
 * Date:		12-15-95
 *
 * Modifications:
 *
 */

#ifndef _NS8390_H
#define _NS8390_H

#include "uif/dbug.h"
#include "net/eth.h"
#include "net/nbuf.h"
#include "net/nif.h"

#if (defined(MCF5204_EVS) || defined(MCF5206_EVS))

/*
 * The MCF5204 EVS board has a screwed up ISA bus interface.
 * The ISA bus on this board only handles 8 bits, 16 bits WILL
 * NOT WORK!!!  In addition, the odd and even addresses on this
 * bus are not sequential.  Instead, even addresses appear at
 * beginning at 0x0000, and odd addresses appear beginning at
 * 0x8000.
 */

typedef struct
{
	NATURAL16	CR;
	union
	{
		struct
		{
			/* Even registers */
			NATURAL16	CLDA1;		/* CLDA1 (rd) PSTOP (wr) */
			NATURAL16	TSR;		/* TSR (rd) TPSR (wr) */
			NATURAL16	FIFO;		/* FIFO (rd) TBCR1 (wr) */
			NATURAL16	CRDA0;		/* CRDA0 (rd) RSAR0 (wr) */
			NATURAL16	RBCR0;		/* Remote Byte Count 0 (wr) */
			NATURAL16	RSR;		/* RSR (rd) RCR (wr) */
			NATURAL16	CNTR1;		/* CNTR1 (rd) DCR (wr) */

			NATURAL16	DATAPORT;

			NATURAL16	reserved[(0x10000-0x0012)/2];

			/* Odd registers */
			NATURAL16	CLDA0;		/* CLDA0 (rd) PSTART (wr) */
			NATURAL16	BNRY;		/* Boundary pointer (rd wr) */
			NATURAL16	NCR;		/* NCR (rd) TBCR0 (wr) */
			NATURAL16	ISR;		/* Interrupt Status Register (rd wr) */
			NATURAL16	CRDA1;		/* CRDA1 (rd) RSAR1 (wr) */
			NATURAL16	RBCR1;		/* Remote Byte Count 1 (wr) */
			NATURAL16	CNTR0;		/* CNTR0 (rd) TCR (wr) */
			NATURAL16	CNTR2;		/* CNTR2 (rd) IMR (wr) */
		} page0;
		struct
		{
			/* Even registers */
			NATURAL16	PAR1;		/* Physical Address Byte 1 */
			NATURAL16	PAR3;		/* Physical Address Byte 3 */
			NATURAL16	PAR5;		/* Physical Address Byte 5 */
			NATURAL16	MAR0;		/* Multicast Address Byte 0 */
			NATURAL16	MAR2;		/* Multicast Address Byte 2 */
			NATURAL16	MAR4;		/* Multicast Address Byte 4 */
			NATURAL16	MAR6;		/* Multicast Address Byte 6 */

			NATURAL16	reserved[(0x10000-0x0010)/2];

			/* Odd registers */
			NATURAL16	PAR0;		/* Physical Address Byte 0 */
			NATURAL16	PAR2;		/* Physical Address Byte 2 */
			NATURAL16	PAR4;		/* Physical Address Byte 4 */
			NATURAL16	CURR;		/* Current Page Register (rd wr) */
			NATURAL16	MAR1;		/* Multicast Address Byte 1 */
			NATURAL16	MAR3;		/* Multicast Address Byte 3 */
			NATURAL16	MAR5;		/* Multicast Address Byte 5 */
			NATURAL16	MAR7;		/* Multicast Address Byte 7 */
		} page1;
		struct
		{
			/* Even registers */
			NATURAL16	PSTOP;		/* PSTOP (rd) CLDA1 (wr) */
			NATURAL16	TPSR;		/* Transmit Page Start Address (rd) */
			NATURAL16	ACU;		/* Address Counter Upper */
			NATURAL16	reserved0;
			NATURAL16	reserved2;
			NATURAL16	RCR;		/* Receive Configuration Register (rd) */
			NATURAL16	DCR;		/* Data Configuration Register (rd) */

			NATURAL16	reserved[(0x10000-0x0010)/2];

			/* Odd registers */
			NATURAL16	PSTART;		/* PSTART (rd) CLDA0 (wr) */
			NATURAL16	RNPP;		/* Remote Next Packet Pointer */
			NATURAL16	LNPP;		/* Local Next Packet Pointer */
			NATURAL16	ACL;		/* Address Counter Lower */
			NATURAL16	reserved1;
			NATURAL16	reserved3;
			NATURAL16	TCR;		/* Transmit Configuration Register (rd) */
			NATURAL16	IMR;		/* Interrupt Mask Register (rd) */
		} page2;
	} regs;
} NS8390;

#else

typedef struct
{
	BYTE	CR;		/* Command Register (rd wr) */
	union
	{
		struct
		{
			BYTE	CLDA0;		/* CLDA0 (rd) PSTART (wr) */
			BYTE	CLDA1;		/* CLDA1 (rd) PSTOP (wr) */
			BYTE	BNRY;		/* Boundary pointer (rd wr) */
			BYTE	TSR;		/* TSR (rd) TPSR (wr) */
			BYTE	NCR;		/* NCR (rd) TBCR0 (wr) */
			BYTE	FIFO;		/* FIFO (rd) TBCR1 (wr) */
			BYTE	ISR;		/* Interrupt Status Register (rd wr) */
			BYTE	CRDA0;		/* CRDA0 (rd) RSAR0 (wr) */
			BYTE	CRDA1;		/* CRDA1 (rd) RSAR1 (wr) */
			BYTE	RBCR0;		/* Remote Byte Count 0 (wr) */
			BYTE	RBCR1;		/* Remote Byte Count 1 (wr) */
			BYTE	RSR;		/* RSR (rd) RCR (wr) */
			BYTE	CNTR0;		/* CNTR0 (rd) TCR (wr) */
			BYTE	CNTR1;		/* CNTR1 (rd) DCR (wr) */
			BYTE	CNTR2;		/* CNTR2 (rd) IMR (wr) */
			BYTE	DATAPORT;
		} page0;
		struct
		{
			BYTE	PAR0;		/* Physical Address Byte 0 */
			BYTE	PAR1;		/* Physical Address Byte 1 */
			BYTE	PAR2;		/* Physical Address Byte 2 */
			BYTE	PAR3;		/* Physical Address Byte 3 */
			BYTE	PAR4;		/* Physical Address Byte 4 */
			BYTE	PAR5;		/* Physical Address Byte 5 */
			BYTE	CURR;		/* Current Page Register (rd wr) */
			BYTE	MAR0;		/* Multicast Address Byte 0 */
			BYTE	MAR1;		/* Multicast Address Byte 1 */
			BYTE	MAR2;		/* Multicast Address Byte 2 */
			BYTE	MAR3;		/* Multicast Address Byte 3 */
			BYTE	MAR4;		/* Multicast Address Byte 4 */
			BYTE	MAR5;		/* Multicast Address Byte 5 */
			BYTE	MAR6;		/* Multicast Address Byte 6 */
			BYTE	MAR7;		/* Multicast Address Byte 7 */
		} page1;
		struct
		{
			BYTE	PSTART;		/* PSTART (rd) CLDA0 (wr) */
			BYTE	PSTOP;		/* PSTOP (rd) CLDA1 (wr) */
			BYTE	RNPP;		/* Remote Next Packet Pointer */
			BYTE	TPSR;		/* Transmit Page Start Address (rd) */
			BYTE	LNPP;		/* Local Next Packet Pointer */
			BYTE	ACU;		/* Address Counter Upper */
			BYTE	ACL;		/* Address Counter Lower */
			BYTE	reserved0;
			BYTE	reserved1;
			BYTE	reserved2;
			BYTE	reserved3;
			BYTE	RCR;		/* Receive Configuration Register (rd) */
			BYTE	TCR;		/* Transmit Configuration Register (rd) */
			BYTE	DCR;		/* Data Configuration Register (rd) */
			BYTE	IMR;		/* Interrupt Mask Register (rd) */
		} page2;
		struct
		{
			/* This page should never be modified */
			BYTE	reserved[15];
		} page3;
	} regs;
} NS8390;

#endif

/*
 * Bit pattern definitions for the chipset.
 */

/* Command Register, CR (rd wr) */
#define NS8390_CR_STP	(0x01)
#define NS8390_CR_STA	(0x02)
#define NS8390_CR_TXP	(0x04)
#define NS8390_CR_RD_RD	(0x08)
#define NS8390_CR_RD_WR	(0x10)
#define NS8390_CR_RD_SP	(0x11)
#define NS8390_CR_RD_AB	(0x20)
#define NS8390_CR_PAGE0	(0x00)
#define NS8390_CR_PAGE1	(0x40)
#define NS8390_CR_PAGE2	(0x80)
#define NS8390_CR_PAGE3	(0xC0)

/* Transmit Control Register, TCR (wr) */
#define NS8390_TCR_CRC		(0x01)
#define NS8390_TCR_LOOP_NO	(0x00)
#define NS8390_TCR_LOOP_I0	(0x02)
#define NS8390_TCR_LOOP_E1	(0x04)
#define NS8390_TCR_LOOP_E0	(0x06)
#define NS8390_TCR_ATD		(0x08)
#define NS8390_TCR_OFST		(0x10)

/* Data Configuration Register, DCR (wr) */
#define NS8390_DCR_WTS		(0x01)
#define NS8390_DCR_BOS		(0x02)
#define NS8390_DCR_LAS		(0x04)
#define NS8390_DCR_LS		(0x08)
#define NS8390_DCR_AR		(0x10)
#define NS8390_DCR_FT_2B	(0x00)
#define NS8390_DCR_FT_4B	(0x20)
#define NS8390_DCR_FT_8B	(0x40)
#define NS8390_DCR_FT_12B	(0x60)
#define NS8390_DCR_FT_1W	(0x00)
#define NS8390_DCR_FT_2W	(0x20)
#define NS8390_DCR_FT_4W	(0x40)
#define NS8390_DCR_FT_6W	(0x60)

/* Receive Configuration Register, RCR (wr) */
#define NS8390_RCR_SEP		(0x01)
#define NS8390_RCR_AR		(0x02)
#define NS8390_RCR_AB		(0x04)
#define NS8390_RCR_AM		(0x08)
#define NS8390_RCR_PRO		(0x10)
#define NS8390_RCR_MON		(0x20)

/* Interrupt Status Register, ISR (rd wr) */
#define NS8390_ISR_PRX		(0x01)
#define NS8390_ISR_PTX		(0x02)
#define NS8390_ISR_RXE		(0x04)
#define NS8390_ISR_TXE		(0x08)
#define NS8390_ISR_OVW		(0x10)
#define NS8390_ISR_CNT		(0x20)
#define NS8390_ISR_RDC		(0x40)
#define NS8390_ISR_RST		(0x80)

/* Interrupt Mask Register, IMR (wr) */
#define NS8390_IMR_PRXE		(0x01)
#define NS8390_IMR_PTXE		(0x02)
#define NS8390_IMR_RXEE		(0x04)
#define NS8390_IMR_TXEE		(0x08)
#define NS8390_IMR_OVWE		(0x10)
#define NS8390_IMR_CNTE		(0x20)
#define NS8390_IMR_RDCE		(0x40)

/* Received Status Register, RSR, and Packet Status Byte */
#define NS8390_RSR_PRX		(0x01)
#define NS8390_RSR_CRC		(0x02)
#define NS8390_RSR_FAE		(0x04)
#define NS8390_RSR_FO		(0x08)
#define NS8390_RSR_MPA		(0x10)
#define NS8390_RSR_PHY		(0x20)
#define NS8390_RSR_DIS		(0x40)
#define NS8390_RSR_DFR		(0x80)

/* Transmit Status Register, TSR (rd) */
#define NS8390_TSR_PTX		(0x01)
#define NS8390_TSR_COL		(0x04)
#define NS8390_TSR_ABT		(0x08)
#define NS8390_TSR_CRS		(0x10)
#define NS8390_TSR_FU		(0x20)
#define NS8390_TSR_CDH		(0x40)
#define NS8390_TSR_OWC		(0x80)

/*******************************************************************/
int
ns8390_init (NIF *, void *, int);

int
ns8390_handler (NS8390 *, NIF *);

/*******************************************************************/

#endif /* _NS8390_H */
