/*
 * File:		icmp.h
 * Purpose:		Handle Internet Control Message Protocol packets.
 *
 * Notes:		See RFC 792 "Internet Control Message Protocol"
 *				for more details.
 *
 * Date:		11-8-95
 *
 * Modifications:
 *
 */

#ifndef _ICMP_H
#define _ICMP_H

#include "ip.h"
#include "nif.h"

typedef struct
{
	UINT32	unused;
	UINT8	ih_dg;
} icmp_dest_unreachable;
#define ICMP_DEST_UNREACHABLE		(3)	/* type */
#define ICMP_NET_UNREACHABLE		(0)	/* code */
#define ICMP_HOST_UNREACHABLE		(1)
#define ICMP_PROTOCOL_UNREACHABLE	(2)
#define ICMP_PORT_UNREACHABLE		(3)
#define ICMP_FRAG_NEEDED			(4)
#define ICMP_ROUTE_FAILED			(5)

typedef struct
{
	UINT32	unused;
	UINT8	ih_dg;
} icmp_time_exceeded;
#define ICMP_TIME_EXCEEDED			(11)	/* type */
#define ICMP_TTL_EXCEEDED			(0)		/* code */
#define ICMP_FRAG_TIME_EXCEEDED		(1)

typedef struct
{
	UINT8	pointer;
	UINT8	unused1;
	UINT16	unused2;
	UINT8	ih_dg;
} icmp_parameter_problem;
#define ICMP_PARAMETER_PROBLEM		(12)	/* type */
#define ICMP_POINTER				(0)		/* code -- not */

typedef struct
{
	UINT32	unused;
	UINT8	ih_dg;
} icmp_source_quench;
#define ICMP_SOURCE_QUENCH			(4)		/* type */

typedef struct
{
	UINT32	gateway_addr;
	UINT8	ih_dg;
} icmp_redirect;
#define ICMP_REDIRECT				(5)		/* type */
#define ICMP_REDIRECT_NET			(0)		/* code */
#define ICMP_REDIRECT_HOST			(1)
#define ICMP_REDIRECT_TOS_NET		(2)
#define ICMP_REDIRECT_TOS_HOST		(3)

typedef struct
{
	UINT16	identifier;
	UINT16	sequence;
	UINT8	data;
} icmp_echo;
#define ICMP_ECHO					(8)		/* type */
#define ICMP_ECHO_REPLY				(0)		/* type */

typedef struct
{
	UINT16	identifier;
	UINT16	sequence;
} icmp_information;
#define ICMP_INFORMATION_REQUEST	(15)	/* type */
#define ICMP_INFORMATION_REPLY		(16)	/* type */

typedef struct
{
	UINT16	identifier;
	UINT16	sequence;
	UINT32	originate_ts;
	UINT32	receive_ts;
	UINT32	transmit_ts;
} icmp_timestamp;
#define ICMP_TIMESTAMP				(13)	/* type */
#define ICMP_TIMESTAMP_REPLY		(14)	/* type */


typedef struct
{
	UINT8	type;
	UINT8	code;
	UINT16	chksum;
	union
	{
		icmp_dest_unreachable		dest_unreachable;
		icmp_source_quench			source_quench;
		icmp_redirect				redirect;
		icmp_time_exceeded			time_exceeded;
		icmp_parameter_problem		parameter_problem;
		icmp_timestamp				timestamp;
		icmp_information			information;
		icmp_echo					echo;
	} msg;
} icmp_message;

void
icmp_handler(NIF *, ip_frame_hdr *, UINT8 *, int);

#endif	/* _ICMP_H */
